#!/usr/bin/env python3
import shutil
import site
from pathlib import Path
from subprocess import run
from sys import argv

import torch
from torch2hpvm import BinDataset, ModelExporter
from torch.nn import Module

site.addsitedir(Path(__file__).parent.parent.absolute())
import dnn

benchmarks = {
    "lenet_mnist": (dnn.LeNet, 1, 28, 1000),
    "alexnet_cifar10": (dnn.AlexNet, 3, 32, 500),
    "alexnet2_cifar10": (dnn.AlexNet2, 3, 32, 500),
    "alexnet_imagenet": (dnn.AlexNetImageNet, 3, 224, 500),
    "mobilenet_cifar10": (dnn.MobileNet, 3, 32, 500),
    "resnet18_cifar10": (dnn.ResNet18, 3, 32, 500),
    "resnet50_imagenet": (dnn.ResNet50, 3, 224, 25),
    "vgg16_cifar10": (dnn.VGG16Cifar10, 3, 32, 500),
    "vgg16_cifar100": (dnn.VGG16Cifar100, 3, 32, 500),
    "vgg16_imagenet": (dnn.VGG16ImageNet, 3, 224, 10),
}
self_folder = Path(__file__).parent
netname = argv[1]
model_cls, nch, img_size, batch_size = benchmarks[netname]
codegen_dir = Path(f"./{netname}")
print(f"Generating {netname} to {codegen_dir}")
if codegen_dir.exists():
    shutil.rmtree(codegen_dir)

params = self_folder / "../../model_params" / netname
dataset_shape = 5000, nch, img_size, img_size
bin_tuneset = BinDataset(
    params / "tune_input.bin", params / "tune_labels.bin", dataset_shape
)
bin_testset = BinDataset(
    params / "test_input.bin", params / "test_labels.bin", dataset_shape
)
model: Module = model_cls()
checkpoint = self_folder / "../model_params/pytorch" / f"{netname}.pth.tar"
model.load_state_dict(torch.load(checkpoint.as_posix()))
print(model)

build_dir = codegen_dir / "build"
target_binary = build_dir / netname
conf_file = self_folder / "../hpvm-c/benchmarks" / netname / "data/tuner_confs.txt"
exporter = ModelExporter(
    model, bin_tuneset, bin_testset, codegen_dir, config_file=conf_file
)
exporter.generate(batch_size=batch_size).compile(target_binary, build_dir)
run([str(target_binary), "test"], check=True)
